@@ -1,10 +1,12 @@
-package Algorithm::CheckDigits::M11_002;
+package Algorithm::CheckDigits::M97_002;
 
 use 5.006;
 use strict;
 use warnings;
 use integer;
 
+use Math::BigInt;
+
 require Exporter;
 
 our @ISA = qw(Exporter Algorithm::CheckDigits);
@@ -24,6 +26,15 @@ our @EXPORT_OK = ( 'new', @{ $EXPORT_TAGS{'all'} } );
 
 our @EXPORT = ();
 
+my %subst = (
+	A => 10, B => 11, C => 12, D => 13, E => 14,
+	F => 15, G => 16, H => 17, I => 18, J => 19,
+	K => 20, L => 21, M => 22, N => 23, O => 24,
+	P => 25, Q => 26, R => 27, S => 28, T => 29,
+	U => 30, V => 31, W => 32, X => 33, Y => 34,
+	Z => 35,
+);
+
 sub new {
 	my $proto = shift;
 	my $type  = shift;
@@ -35,33 +46,32 @@ sub new {
 
 sub is_valid {
 	my ($self,$number) = @_;
-	if ($number =~ /^(.+)(.)$/) {
-		return uc($2) eq $self->_compute_checkdigit($1);
+	if ($number =~ /^[A-Z]{2}(\d\d)\s?[\d ]{0,37}$/i) {
+		return $1 eq $self->_compute_checkdigit($number);
 	}
 	return ''
 } # is_valid()
 
 sub complete {
 	my ($self,$number) = @_;
-	if ($number =~ /^[0-9]+$/) {
-		my $cd = $self->_compute_checkdigit($number);
-		return $number . $cd unless 0 > $cd;
+	if ($number =~ /^([A-Z]{2})(00)(\s?[\d ]{0,37})$/i) {
+		return $1 . $self->_compute_checkdigit($number) . $3;
 	}
 	return '';
 } # complete()
 
 sub basenumber {
 	my ($self,$number) = @_;
-	if ($number =~ /^(.+)(.)$/) {
-		return $1 if (uc($2) eq $self->_compute_checkdigit($1));
+	if ($number =~ /^([A-Z]{2})(\d\d)(\s?[\d ]{0,37})$/i) {
+		return $1.'00'.$3 if ($2 eq $self->_compute_checkdigit($number));
 	}
 	return '';
 } # basenumber()
 
 sub checkdigit {
 	my ($self,$number) = @_;
-	if ($number =~ /^(.+)(.)$/) {
-		return $2 if (uc($2) eq $self->_compute_checkdigit($1));
+	if ($number =~ /^([A-Z]{2})(\d\d)(\s?[\d ]{0,37})$/i) {
+		return $2 if ($2 eq $self->_compute_checkdigit($number));
 	}
 	return '';
 } # checkdigit()
@@ -70,21 +80,12 @@ sub _compute_checkdigit {
 	my $self   = shift;
 	my $number = shift;
 
-	if ($number =~ /^[-0-9]+$/) {
+	$number =~ s/\s//g;
 
-		$number =~ s/-//g;
-		my @digits = split(//,$number);
-		my $sum    = 0;
-		my $weight = 2;
-
-		for (my $i = 0; $i <= $#digits; $i++) {
-
-			$sum += $weight * $digits[$i];
-			++$weight;
-
-		}
-		$sum %= 11;
-		return 10 == $sum ? -1 : $sum;
+	if ($number =~ /^([A-Z])([A-Z])..([ \d]{0,37})$/i) {
+		my $bignum = Math::BigInt->new($3.$subst{$1}.$subst{$2}.'00');
+		my $mod    = $bignum % 97;
+		return sprintf("%2.2d",(98 - $mod));
 	}
 	return -1;
 } # _compute_checkdigit()
@@ -96,26 +97,27 @@ __END__
 
 =head1 NAME
 
-CheckDigits::M11_002 - compute check digits for PZN (DE)
+CheckDigits::M97_002 - compute check digits for International Bank
+Account Number (IBAN)
 
 =head1 SYNOPSIS
 
   use CheckDigits;
 
-  $pzn = CheckDigits('pzn');
+  $iban = CheckDigits('iban');
 
-  if ($pzn->is_valid('4877800')) {
+  if ($iban->is_valid('DE88 2008 0000 09703 7570 0')) {
 	# do something
   }
 
-  $cn = $pzn->complete('487780');
-  # $cn = '4877800'
+  $cn = $iban->complete('DE00 2008 0000 09703 7570 0');
+  # $cn = 'DE88 2008 0000 09703 7570 0'
 
-  $cd = $pzn->checkdigit('4877800');
-  # $cd = '0'
+  $cd = $iban->checkdigit('DE88 2008 0000 09703 7570 0');
+  # $cd = '88'
 
-  $bn = $pzn->basenumber('4877800');
-  # $bn = '487780'
+  $bn = $iban->basenumber('DE88 2008 0000 09703 7570 0');
+  # $bn = 'DE00 2008 0000 09703 7570 0'
   
 =head1 DESCRIPTION
 
@@ -123,24 +125,24 @@ CheckDigits::M11_002 - compute check digits for PZN (DE)
 
 =over 4
 
+=item 0
+
+The IBAN number must be prepared. The first two letters and the
+checksum will be moved to the right end. The letters are substituted
+according to the substitute table and the checksum is set to '00'.
+
 =item 1
 
-From left to right beginning with the first position all digits are
-multiplied with 2,3,4,...
+The whole number (without checksum) is taken modulo 97.
 
 =item 2
 
-The sum of all products is computed.
+The checksum is difference between 98 and the sum of step 2.
 
 =item 3
 
-The checkdigit ist the sum of step 2 taken modulo 11.
-
-=item 4
-
-If the checkdigit is '10' the whole number is not taken as a PZN.
-
-=over 8
+If the checksum is smaller then 10, a leading zero will be
+supplemented.
 
 =back
 
@@ -172,7 +174,7 @@ Return '' otherwise.
 
 =item checkdigit($number)
 
-Returns the checkdigit of C<$number> if C<$number> has a valid check
+Returns the checkdigits of C<$number> if C<$number> has a valid check
 digit.
 
 Return '' otherwise.
